package sun.security.jgss.krb5;

import sun.security.jgss.GSSToken;
import sun.security.util.DerValue;
import java.io.InputStream;
import sun.security.krb5.Credentials;
import org.ietf.jgss.GSSException;
import java.io.IOException;
import sun.security.krb5.KrbException;
import sun.security.krb5.EncryptionKey;
import java.security.AccessController;
import sun.security.action.GetBooleanAction;
import sun.security.krb5.KrbApReq;
import sun.security.krb5.KrbApRep;

class AcceptSecContextToken extends InitialToken
{
    private KrbApRep apRep;
    
    public AcceptSecContextToken(Krb5Context krb5Context, KrbApReq incomingReq) throws KrbException, IOException, GSSException {
        this.apRep = null;
        int booleanValue = (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.security.krb5.acceptor.subkey"));
        boolean useSeqNumber = true;
        EncryptionKey subKey = null;
        if (!(booleanValue == 0)) {
            subKey = new EncryptionKey(incomingReq.getCreds().getSessionKey());
            krb5Context.setKey(2, subKey);
        }
        this.apRep = new KrbApRep(incomingReq, (boolean)useSeqNumber, subKey);
        krb5Context.resetMySequenceNumber((int)this.apRep.getSeqNumber());
    }
    
    public AcceptSecContextToken(Krb5Context krb5Context, Credentials tgtCreds, KrbApReq outgoingReq, InputStream in) throws IOException, GSSException, KrbException {
        this.apRep = null;
        if ((in.read() << 8 | in.read()) != 0x200) {
            throw new GSSException(10, -1, "AP_REP token id does not match!");
        }
        KrbApRep krbApRep = new KrbApRep(new DerValue(in).toByteArray(), tgtCreds, outgoingReq);
        EncryptionKey subKey = krbApRep.getSubKey();
        if (subKey != null) {
            krb5Context.setKey(2, subKey);
        }
        Integer seqNumber = krbApRep.getSeqNumber();
        krb5Context.resetPeerSequenceNumber((seqNumber != null) ? ((int)seqNumber) : 0);
        return;
    }
    
    @Override
    public final byte[] encode() throws IOException {
        byte[] message = this.apRep.getMessage();
        byte[] dest = new byte[2 + message.length];
        GSSToken.writeInt(512, dest, 0);
        System.arraycopy(message, 0, dest, 2, message.length);
        return dest;
    }
}
